@@ -60,6 +60,9 @@ public:
       programDir.cd(".web2sms");
       break;
     }*/
+
+    programDir.mkdir(".web2sms");
+    programDir.cd(".web2sms");
   }
 
   // Properties
@@ -88,8 +91,7 @@ Accounts::Accounts(QObject* parent /* = 0 */)
 Accounts::~Accounts()
 {
   // Clean up
-  foreach(ProviderPlugin* plugin, d->providers)
-    delete plugin;
+  qDeleteAll(d->providers);
   d->providers.clear();
 
   // Those providers which still exist get deleted now
@@ -289,10 +291,10 @@ ProviderInterface* Accounts::providerForAlias(const QString& alias)
     return iter.value();
 
   // Get the provider id
-  QString providerId = d->accounts.value(alias).providerId();
+  QUuid providerId = d->accounts.value(alias).providerId();
 
   // Make sure the providerId is valid (so is the alias)
-  if ( providerId.isEmpty() )
+  if ( providerId.isNull() )
     return NULL;
 
   // Create a new instance of the provider
@@ -341,7 +343,7 @@ ProviderInfo Accounts::providerInfoForAlias(const QString& alias) const
     foreach(const ProviderInfo& info, infoList)
     {
       // If the id is the searched one, we're done
-      if ( info.providerId == config.providerId() )
+      if ( info.providerId() == config.providerId() )
         return info;
     }
   }
