@@ -257,7 +257,7 @@ void o2GermanyPrivate::handleSmsCenter(QNetworkReply* reply)
     freeSMSCount = freeSmsExp.cap(1).toInt();
 
     // Emit how many free-sms we have left
-    emit self->freeSmsCountChanged( freeSMSCount );
+    emit self->balanceReply( tr("%n free SMS", "", freeSMSCount) );
   }
 
   // Try to determine how many free sms we have left
@@ -297,15 +297,6 @@ void o2GermanyPrivate::handleSmsCenter(QNetworkReply* reply)
   }
   else
   {
-#ifdef Q_WS_MAEMO_5
-    QFile file("/home/user/MyDocs/o2sms_debug_handleSmsCenter.html");
-    if ( file.open(QIODevice::Text | QIODevice::WriteOnly) )
-    {
-      file.write(readData);
-      file.close();
-    }
-#endif // Q_WS_MAEMO5
-
     // Emit that we could not find the frmSMS
     emit self->error( tr("frmSMS could not be found!") );
   }
@@ -319,13 +310,10 @@ void o2GermanyPrivate::handleSmsCenterSend(QNetworkReply* reply)
   QByteArray readData = reply->readAll();
   QString data = QString::fromUtf8(readData.constData(), readData.size());
 
-  // TODO: ERROR CHECKING!!!!
+  Message message = qvariant_cast<Message>(reply->property("message"));
 
   // Emit a signal that we've sent the message
-  emit self->sendMessageReply(true, sentMessages);
-
-  // Emit a signal that the free sms count may have changed?
-  emit self->freeSmsCountChanged( --freeSMSCount );
+  emit self->sendMessageReply(true, message);
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -464,20 +452,6 @@ bool o2Germany::showAccountSettingsDialog(QWidget* parent)
 
 //////////////////////////////////////////////////////////////////////////
 
-QList<QAction*> o2Germany::actions() const
-{
-  return QList<QAction*>();
-}
-
-//////////////////////////////////////////////////////////////////////////
-
-bool o2Germany::hasLoginData() const
-{
-  return !d->loginName.isEmpty();
-}
-
-//////////////////////////////////////////////////////////////////////////
-
 bool o2Germany::isLoggedIn() const
 {
   // Simple .. heh?
@@ -486,14 +460,7 @@ bool o2Germany::isLoggedIn() const
 
 //////////////////////////////////////////////////////////////////////////
 
-int o2Germany::freeSMSCount() const
-{
-  return d->freeSMSCount;
-}
-
-//////////////////////////////////////////////////////////////////////////
-
-void o2Germany::login()
+CallbackType o2Germany::login()
 {
   // Start the first request
   QNetworkRequest request = d->prepareRequest();
@@ -501,11 +468,13 @@ void o2Germany::login()
 
   // Start the pre-login
   d->get(request, o2GermanyPrivate::PreLogin);
+
+  return Callback_Async;
 }
 
 //////////////////////////////////////////////////////////////////////////
 
-void o2Germany::logout()
+CallbackType o2Germany::logout()
 {
   // Start the logout request
   QNetworkRequest request = d->prepareRequest();
@@ -513,16 +482,18 @@ void o2Germany::logout()
 
   // Start the logout request
   d->get(request, o2GermanyPrivate::Logout);
+
+  return Callback_Async;
 }
 
 //////////////////////////////////////////////////////////////////////////
 
-int o2Germany::sendMessage(const QList<ContactInfo>& receivers, const QString& text)
+void o2Germany::sendMessage(const Message& message)
 {
   QString smsTo;
 
   // Merge the numbers
-  foreach(const ContactInfo& receiver, receivers)
+  foreach(const ContactInfo& receiver, message.receivers())
   {
     if ( !smsTo.isEmpty() )
       smsTo += ", ";
@@ -542,8 +513,8 @@ int o2Germany::sendMessage(const QList<ContactInfo>& receivers, const QString& t
   QByteArray formData;
 
   // Fill the remaining fields
-  d->sendFormDefaultValues.insert("SMSTo", smsTo.toUtf8().toPercentEncoding());
-  d->sendFormDefaultValues.insert("SMSText", text.toUtf8().toPercentEncoding());
+  d->sendFormDefaultValues.insert("SMSTo", smsTo.toAscii().toPercentEncoding());
+  d->sendFormDefaultValues.insert("SMSText", message.text().toAscii().toPercentEncoding());
 
   // Fill the default values
   QHashIterator<QString, QByteArray> iter(d->sendFormDefaultValues);
@@ -557,16 +528,31 @@ int o2Germany::sendMessage(const QList<ContactInfo>& receivers, const QString& t
     formData += iter.key().toUtf8() + '=' + iter.value();
   }
 
-  d->post(request, formData, o2GermanyPrivate::SmsCenterSend);
+  QNetworkReply* reply = d->post(request, formData, o2GermanyPrivate::SmsCenterSend);
+  reply->setProperty("message", QVariant::fromValue(message));
+}
 
-  // Return a new message id
-  return d->sentMessages++;
+//////////////////////////////////////////////////////////////////////////
+
+void o2Germany::updateBalance()
+{
+  // Nothing, this gets done due login
+}
+
+//////////////////////////////////////////////////////////////////////////
+
+QList<MessageType> o2Germany::messageTypes() const
+{
+  return QList<MessageType>()
+      << MessageType(tr("Standard"), "standard", QIcon(), QString(), 160, 2000, -1);
 }
 
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 
+#define O2_GERMANY_UUID "7f1c4598-e675-4bed-ae64-9d35485f0b34"
+
 class o2GermanyProviderPlugin : public QObject
                               , public ProviderPlugin
 {
@@ -576,7 +562,7 @@ class o2GermanyProviderPlugin : public QObject
 public:
   o2GermanyProviderPlugin()
   {
-    infoList << ProviderInfo(tr("o2 Germany"), "o2_germany", QIcon(":/providers/o2_germany/o2_germany.png"), QLocale::Germany);
+    infoList << ProviderInfo(tr("o2 Germany"), O2_GERMANY_UUID, QIcon(":/providers/o2_germany/o2_germany.png"), QLocale::Germany);
   }
 
   QList<ProviderInfo> info() const
@@ -584,9 +570,9 @@ public:
     return infoList;
   }
 
-  ProviderInterface* createProvider(const QString& name) const
+  ProviderInterface* createProvider(const QUuid& uuid) const
   {
-    if ( name == "o2_germany" )
+    if ( uuid == O2_GERMANY_UUID )
       return new o2Germany;
     
     return NULL;
